@@ -20,57 +20,75 @@
 #'
 #' Evaluate an expression asynchronously in a new background R process or
 #'     persistent daemon (local or remote). This function will return
-#'     immediately with a 'mirai', which will resolve to the evaluated result
-#'     once complete.
+#'     immediately with a \sQuote{mirai}, which will resolve to the evaluated
+#'     result once complete.
 #'
 #' @param .expr an expression to evaluate asynchronously (of arbitrary length,
-#'     wrapped in \{ \} where necessary), \strong{or} a language object passed
-#'     by \link{name}.
-#' @param ... (optional) named arguments (name = value pairs) specifying
-#'     objects referenced in '.expr'. These are placed in the global environment
-#'     of the evaluation process, unlike those supplied to '.args' below.
-#' @param .args (optional) \strong{either} a list of objects passed by
-#'     \link{name} (found in the current scope), \strong{or else} a list of
-#'     name = value pairs, as in '...'. These remain local to the evaluation
-#'     environment.
+#'     wrapped in \{ \} where necessary), \strong{or else} a pre-constructed
+#'     language object.
+#' @param ... (optional) \strong{either} named arguments (name = value pairs)
+#'     specifying objects referenced, but not defined, in \sQuote{.expr},
+#'     \strong{or} an environment containing such objects. See
+#'     \sQuote{evaluation} section below.
+#' @param .args (optional) \strong{either} a named list specifying objects
+#'     referenced, but not defined, in \sQuote{.expr}, \strong{or} an
+#'     environment containing such objects. These objects will remain local to
+#'     the evaluation environment as opposed to those supplied in \sQuote{...}
+#'     above - see \sQuote{evaluation} section below.
 #' @param .timeout [default NULL] for no timeout, or an integer value in
-#'     milliseconds. A mirai will resolve to an 'errorValue' 5 (timed out) if
-#'     evaluation exceeds this limit.
+#'     milliseconds. A mirai will resolve to an \sQuote{errorValue} 5 (timed
+#'     out) if evaluation exceeds this limit.
 #' @param .compute [default 'default'] character value for the compute profile
 #'     to use (each compute profile has its own independent set of daemons).
 #'
-#' @return A 'mirai' object.
+#' @return A \sQuote{mirai} object.
 #'
-#' @details This function will return a 'mirai' object immediately.
+#' @details This function will return a \sQuote{mirai} object immediately.
 #'
 #'     The value of a mirai may be accessed at any time at \code{$data}, and
-#'     if yet to resolve, an 'unresolved' logical NA will be returned instead.
+#'     if yet to resolve, an \sQuote{unresolved} logical NA will be returned
+#'     instead.
 #'
 #'     \code{\link{unresolved}} may be used on a mirai, returning TRUE if a
-#'     'mirai' has yet to resolve and FALSE otherwise. This is suitable for use
-#'     in control flow statements such as \code{while} or \code{if}.
+#'     \sQuote{mirai} has yet to resolve and FALSE otherwise. This is suitable
+#'     for use in control flow statements such as \code{while} or \code{if}.
 #'
 #'     Alternatively, to call (and wait for) the result, use
-#'     \code{\link{call_mirai}} on the returned mirai. This will block until the
-#'     result is returned.
+#'     \code{\link{call_mirai}} on the returned \sQuote{mirai}. This will block
+#'     until the result is returned.
 #'
-#'     The expression '.expr' will be evaluated in a separate R process in a
-#'     clean environment (not the global environment), consisting only of the
-#'     objects in the list supplied to '.args', with the named objects passed as
-#'     '...' assigned to the global environment of that process.
+#'     Specify \sQuote{.compute} to send the mirai using a specific compute
+#'     profile (if previously created by \code{\link{daemons}}), otherwise leave
+#'     as \sQuote{default}.
 #'
-#'     Specify '.compute' to send the mirai using a specific compute profile (if
-#'     previously created by \code{\link{daemons}}), otherwise leave as 'default'.
+#' @section Evaluation:
+#'
+#'     The expression \sQuote{.expr} will be evaluated in a separate R process
+#'     in a clean environment (not the global environment), consisting only of
+#'     the objects in the list or environment supplied to \sQuote{.args}, with
+#'     the named objects passed as \sQuote{...} (from the environment if one was
+#'     supplied) assigned to the global environment of that process.
+#'
+#'     For evaluation to occur \emph{as if} in your global environment, supply
+#'     objects to \sQuote{...} rather than \sQuote{.args}. For stricter scoping,
+#'     use \sQuote{.args}, which limits, for example, where variables not
+#'     explicitly passed as arguments to functions are found.
+#'
+#'     As evaluation occurs in a clean environment, all undefined objects must
+#'     be supplied though \sQuote{...} and/or \sQuote{.args}, including
+#'     self-defined functions. Functions from a package should use namespaced
+#'     calls such as \code{mirai::mirai()}, or else the package should be loaded
+#'     beforehand as part of \sQuote{.expr}.
 #'
 #' @section Errors:
 #'
 #'     If an error occurs in evaluation, the error message is returned as a
-#'     character string of class 'miraiError' and 'errorValue' (the stack trace
-#'     is available at \code{$stack.trace} on the error object).
-#'     \code{\link{is_mirai_error}} may be used to test for this.
+#'     character string of class \sQuote{miraiError} and \sQuote{errorValue}
+#'     (the stack trace is available at \code{$stack.trace} on the error
+#'     object). \code{\link{is_mirai_error}} may be used to test for this.
 #'
 #'     \code{\link{is_error_value}} tests for all error conditions including
-#'     'mirai' errors, interrupts, and timeouts.
+#'     \sQuote{mirai} errors, interrupts, and timeouts.
 #'
 #' @examples
 #' if (interactive()) {
@@ -84,10 +102,10 @@
 #' Sys.sleep(0.2)
 #' m$data
 #'
-#' # passing existing objects by name via '.args'
+#' # passing the calling environment to '...'
 #' df1 <- data.frame(a = 1, b = 2)
 #' df2 <- data.frame(a = 3, b = 1)
-#' m <- mirai(as.matrix(rbind(df1, df2)), .args = list(df1, df2), .timeout = 1000)
+#' m <- mirai(as.matrix(rbind(df1, df2)), environment(), .timeout = 1000)
 #' call_mirai(m)$data
 #'
 #' # using unresolved()
@@ -104,14 +122,6 @@
 #' }
 #' str(m$data)
 #'
-#' # evaluating scripts using source(local = TRUE) in '.expr'
-#' n <- 10L
-#' file <- tempfile()
-#' cat("r <- rnorm(n)", file = file)
-#' m <- mirai({source(file, local = TRUE); r}, .args = list(file, n))
-#' call_mirai(m)[["data"]]
-#' unlink(file)
-#'
 #' # evaluating scripts using source() in '.expr'
 #' n <- 10L
 #' file <- tempfile()
@@ -120,6 +130,14 @@
 #' call_mirai(m)[["data"]]
 #' unlink(file)
 #'
+#' # use source(local = TRUE) when passing in local variables via '.args'
+#' n <- 10L
+#' file <- tempfile()
+#' cat("r <- rnorm(n)", file = file)
+#' m <- mirai({source(file, local = TRUE); r}, .args = list(file = file, n = n))
+#' call_mirai(m)[["data"]]
+#' unlink(file)
+#'
 #' # passing a language object to '.expr' and a named list to '.args'
 #' expr <- quote(a + b + 2)
 #' args <- list(a = 2, b = 3)
@@ -136,19 +154,32 @@ mirai <- function(.expr, ..., .args = list(), .timeout = NULL, .compute = "defau
 
   expr <- substitute(.expr)
   globals <- list(...)
-  all(nzchar(names(globals))) || stop(._[["named_args"]])
-  arglist <- list(._mirai_globals_. = globals, .expr = if (is.symbol(expr) && is.language(.expr)) .expr else expr)
+  glen <- length(globals)
+  if (glen) {
+    gn <- names(globals)
+    if (is.null(gn)) {
+      glen == 1L && is.environment(globals[[1L]]) || stop(._[["named_args"]])
+      globals <- as.list(globals[[1L]])
+    }
+    all(nzchar(gn)) || stop(._[["named_args"]])
+  }
+  arglist <- list(
+    ._mirai_globals_. = globals,
+    .expr = if (is.symbol(expr) && exists(expr, where = parent.frame()) && is.language(.expr)) .expr else expr
+  )
   if (length(.args))
-    arglist <- c(if (is.null(names(.args))) `names<-`(.args, as.character(substitute(.args)[-1L])) else .args, arglist)
+    arglist <- c(if (is.environment(.args)) as.list.environment(.args) else .args, arglist)
   data <- list2env(arglist, envir = NULL, parent = .GlobalEnv)
 
   envir <- ..[[.compute]]
   if (is.null(envir)) {
     sock <- ephemeral_daemon(local_url())
-    aio <- request(.context(sock), data = data, send_mode = 1L, recv_mode = 1L, timeout = .timeout)
+    aio <- request(.context(sock), data = data,
+                   send_mode = 1L, recv_mode = 1L, timeout = .timeout)
     `attr<-`(.subset2(aio, "aio"), "sock", sock)
   } else {
-    aio <- request_signal(.context(envir[["sock"]]), data = data, cv = envir[["cv"]], send_mode = 3L, recv_mode = 1L, timeout = .timeout)
+    aio <- request_signal(.context(envir[["sock"]]), data = data, cv = envir[["cv"]],
+                          send_mode = 3L, recv_mode = 1L, timeout = .timeout)
   }
 
   `class<-`(aio, c("mirai", "recvAio"))
@@ -157,16 +188,18 @@ mirai <- function(.expr, ..., .args = list(), .timeout = NULL, .compute = "defau
 
 #' Evaluate Everywhere
 #'
-#' Evaluate an expression 'everywhere' on all connected daemons for the
+#' Evaluate an expression \sQuote{everywhere} on all connected daemons for the
 #'     specified compute profile. Designed for performing setup operations
 #'     across daemons or exporting common data, resultant changes to the global
 #'     environment, loaded packages or options are persisted regardless of a
-#'     daemon's 'cleanup' setting.
+#'     daemon's \sQuote{cleanup} setting.
 #'
 #' @inheritParams mirai
 #'
 #' @return Invisible NULL.
 #'
+#' @inheritSection mirai Evaluation
+#'
 #' @examples
 #' if (interactive()) {
 #' # Only run examples in interactive R sessions
@@ -197,9 +230,10 @@ everywhere <- function(.expr, ..., .args = list(), .compute = "default") {
   if (length(envir)) {
 
     expr <- substitute(.expr)
-    .expr <- c(as.expression(if (is.symbol(expr) && is.language(.expr)) .expr else expr), .snapshot)
-    if (length(.args) && is.null(names(.args)))
-      names(.args) <- as.character(substitute(.args)[-1L])
+    .expr <- c(
+      as.expression(if (is.symbol(expr) && is.language(.expr)) .expr else expr),
+      .snapshot
+    )
 
     if (is.null(envir[["sockc"]])) {
       for (i in seq_len(max(stat(envir[["sock"]], "pipes"), envir[["n"]])))
@@ -216,29 +250,30 @@ everywhere <- function(.expr, ..., .args = list(), .compute = "default") {
 
 #' mirai (Call Value)
 #'
-#' \code{call_mirai} retrieves the value of a mirai, waiting for the the
+#' \code{call_mirai} retrieves the value of a \sQuote{mirai}, waiting for the
 #'     asynchronous operation to resolve if it is still in progress.
 #'
-#' @param aio a 'mirai' object.
+#' @param aio a \sQuote{mirai} object.
 #'
-#' @return The passed mirai (invisibly). The retrieved value is stored at
-#'     \code{$data}.
+#' @return The passed \sQuote{mirai} (invisibly). The retrieved value is stored
+#'     at \code{$data}.
 #'
 #' @details This function will wait for the async operation to complete if still
 #'     in progress (blocking).
 #'
-#'     The mirai updates itself in place, so to access the value of a mirai
-#'     \code{x} directly, use \code{call_mirai(x)$data}.
+#'     The \sQuote{mirai} updates itself in place, so to access the value of a
+#'     \sQuote{mirai} \code{x} directly, use \code{call_mirai(x)$data}.
 #'
 #' @inheritSection mirai Errors
 #'
 #' @section Alternatively:
 #'
-#'     The value of a mirai may be accessed at any time at \code{$data}, and
-#'     if yet to resolve, an 'unresolved' logical NA will be returned instead.
+#'     The value of a \sQuote{mirai} may be accessed at any time at
+#'     \code{$data}, and if yet to resolve, an \sQuote{unresolved} logical NA
+#'     will be returned instead.
 #'
-#'     Using \code{\link{unresolved}} on a mirai returns TRUE only if a mirai
-#'     has yet to resolve and FALSE otherwise. This is suitable for use in
+#'     Using \code{\link{unresolved}} on a \sQuote{mirai} returns TRUE only if
+#'     it has yet to resolve and FALSE otherwise. This is suitable for use in
 #'     control flow statements such as \code{while} or \code{if}.
 #'
 #' @examples
@@ -248,15 +283,17 @@ everywhere <- function(.expr, ..., .args = list(), .compute = "default") {
 #' # using call_mirai()
 #' df1 <- data.frame(a = 1, b = 2)
 #' df2 <- data.frame(a = 3, b = 1)
-#' m <- mirai(as.matrix(rbind(df1, df2)), .args = list(df1, df2), .timeout = 1000)
+#' m <- mirai(as.matrix(rbind(df1, df2)), df1 = df1, df2 = df2, .timeout = 1000)
 #' call_mirai(m)$data
 #'
 #' # using unresolved()
-#' m <- mirai({
-#'   res <- rnorm(n)
-#'   res / rev(res)
+#' m <- mirai(
+#'   {
+#'     res <- rnorm(n)
+#'     res / rev(res)
 #'   },
-#'   n = 1e6)
+#'   n = 1e6
+#' )
 #' while (unresolved(m)) {
 #'   cat("unresolved\n")
 #'   Sys.sleep(0.1)
@@ -279,25 +316,31 @@ call_mirai <- call_aio
 #'
 call_mirai_ <- call_aio_
 
-#' mirai (Stop Evaluation)
+#' mirai (Stop)
 #'
-#' Stop evaluation of a mirai that is in progress.
+#' Stops a \sQuote{mirai} if still in progress, causing it to resolve
+#'     immediately to an \sQuote{errorValue} 20 (Operation canceled).
 #'
-#' @param aio a 'mirai' object.
+#' @param aio a \sQuote{mirai} object.
 #'
 #' @return Invisible NULL.
 #'
-#' @details Stops the asynchronous operation associated with the mirai by
-#'     aborting, and then waits for it to complete or to be completely aborted.
-#'     The mirai is then deallocated and attempting to access the value at
-#'     \code{$data} will result in an error.
+#' @details Forces the \sQuote{mirai} to resolve immediately. Has no effect if
+#'     the \sQuote{mirai} has already resolved.
+#'
+#'     If cancellation was successful, the value at \code{$data} will be an
+#'     \sQuote{errorValue} 20 (Operation canceled). Note that in such a case,
+#'     the \sQuote{mirai} has been aborted and the value not retrieved - but any
+#'     ongoing evaluation in the daemon process will continue to completion and
+#'     is not interrupted.
 #'
 #' @examples
 #' if (interactive()) {
 #' # Only run examples in interactive R sessions
 #'
-#' s <- mirai(Sys.sleep(n), n = 5)
-#' stop_mirai(s)
+#' m <- mirai(Sys.sleep(n), n = 5)
+#' stop_mirai(m)
+#' m$data
 #'
 #' }
 #'
@@ -307,19 +350,21 @@ stop_mirai <- stop_aio
 
 #' Query if a mirai is Unresolved
 #'
-#' Query whether a mirai or mirai value remains unresolved. Unlike
-#'     \code{\link{call_mirai}}, this function does not wait for completion.
+#' Query whether a \sQuote{mirai} or \sQuote{mirai} value remains unresolved.
+#'     Unlike \code{\link{call_mirai}}, this function does not wait for
+#'     completion.
 #'
-#' @param aio a 'mirai' object or 'mirai' value stored at \code{$data}.
+#' @param aio a \sQuote{mirai} object or \sQuote{mirai} value stored at
+#'     \code{$data}.
 #'
-#' @return Logical TRUE if 'aio' is an unresolved mirai or mirai value, or
-#'     FALSE otherwise.
+#' @return Logical TRUE if \sQuote{aio} is an unresolved \sQuote{mirai} or
+#'     \sQuote{mirai} value, or FALSE otherwise.
 #'
 #' @details Suitable for use in control flow statements such as \code{while} or
 #'     \code{if}.
 #'
-#'     Note: querying resolution may cause a previously unresolved 'mirai' to
-#'     resolve.
+#'     Note: querying resolution may cause a previously unresolved
+#'     \sQuote{mirai} to resolve.
 #'
 #' @examples
 #' if (interactive()) {
@@ -338,18 +383,19 @@ unresolved <- unresolved
 
 #' Is mirai
 #'
-#' Is the object a 'mirai'.
+#' Is the object a \sQuote{mirai}.
 #'
 #' @param x an object.
 #'
-#' @return Logical TRUE if 'x' is of class 'mirai', FALSE otherwise.
+#' @return Logical TRUE if \sQuote{x} is of class \sQuote{mirai}, FALSE
+#'     otherwise.
 #'
 #' @examples
 #' if (interactive()) {
 #' # Only run examples in interactive R sessions
 #'
 #' df <- data.frame()
-#' m <- mirai(as.matrix(df), .args = list(df))
+#' m <- mirai(as.matrix(df), df = df)
 #' is_mirai(m)
 #' is_mirai(df)
 #'
@@ -367,18 +413,18 @@ is_mirai <- function(x) inherits(x, "mirai")
 #'
 #' @return Logical value TRUE or FALSE.
 #'
-#' @details Is the object a 'miraiError'. When execution in a mirai process
-#'     fails, the error message is returned as a character string of class
-#'     'miraiError' and 'errorValue'. The stack trace is available at
-#'     \code{$stack.trace} on the error object.
+#' @details Is the object a \sQuote{miraiError}. When execution in a
+#'     \sQuote{mirai} process fails, the error message is returned as a
+#'     character string of class \sQuote{miraiError} and \sQuote{errorValue}.
+#'     The stack trace is available at \code{$stack.trace} on the error object.
 #'
-#'     Is the object a 'miraiInterrupt'. When an ongoing mirai is sent a user
-#'     interrupt, the mirai will resolve to an empty character string classed as
-#'     'miraiInterrupt' and 'errorValue'.
+#'     Is the object a \sQuote{miraiInterrupt}. When an ongoing \sQuote{mirai}
+#'     is sent a user interrupt, it will resolve to an empty character string
+#'     classed as \sQuote{miraiInterrupt} and \sQuote{errorValue}.
 #'
-#'     Is the object an 'errorValue', such as a mirai timeout, a 'miraiError' or
-#'     a 'miraiInterrupt'. This is a catch-all condition that includes all
-#'     returned error values.
+#'     Is the object an \sQuote{errorValue}, such as a \sQuote{mirai} timeout,
+#'     a \sQuote{miraiError} or a \sQuote{miraiInterrupt}. This is a catch-all
+#'     condition that includes all returned error values.
 #'
 #' @examples
 #' if (interactive()) {
@@ -431,6 +477,15 @@ print.miraiError <- function(x, ...) {
 
 }
 
+#' @export
+#'
+print.miraiInterrupt <- function(x, ...) {
+
+  cat("'miraiInterrupt' chr \"\"\n", file = stdout())
+  invisible(x)
+
+}
+
 #' @export
 #'
 `$.miraiError` <- function(x, name)
@@ -441,15 +496,6 @@ print.miraiError <- function(x, ...) {
 .DollarNames.miraiError <- function(x, pattern = "")
   grep(pattern, "stack.trace", value = TRUE, fixed = TRUE)
 
-#' @export
-#'
-print.miraiInterrupt <- function(x, ...) {
-
-  cat("'miraiInterrupt' chr \"\"\n", file = stdout())
-  invisible(x)
-
-}
-
 # internals --------------------------------------------------------------------
 
 ephemeral_daemon <- function(url) {
@@ -463,28 +509,18 @@ deparse_safe <- function(x) if (length(x))
 
 deparse_call <- function(call) {
   srcref <- attr(call, "srcref")
-  if (is.null(srcref)) deparse_safe(call) else
-    as.character(srcref)
+  if (is.null(srcref)) deparse_safe(call) else as.character(srcref)
 }
 
-mk_interrupt_error <- function()
-  `class<-`("", c("miraiInterrupt", "errorValue", "try-error"))
+mk_interrupt_error <- function() .miraiInterrupt
 
 mk_mirai_error <- function(e, sc) {
   call <- deparse_safe(.subset2(e, "call"))
   msg <- if (is.null(call) || call == "eval(expr = ._mirai_.[[\".expr\"]], envir = ._mirai_., enclos = NULL)")
-    sprintf("Error: %s\n", .subset2(e, "message")) else
-      sprintf("Error in %s: %s\n", call, .subset2(e, "message"))
-  cat(msg, file = stderr())
-  idx <- which(
-    as.logical(
-      lapply(
-        sc,
-        identical,
-        quote(eval(expr = ._mirai_.[[".expr"]], envir = ._mirai_., enclos = NULL))
-      )
-    )
-  )
+    sprintf("Error: %s", .subset2(e, "message")) else
+      sprintf("Error in %s: %s", call, .subset2(e, "message"))
+  cat(strcat(msg, "\n"), file = stderr())
+  idx <- which(as.logical(lapply(sc, identical, quote(eval(expr = ._mirai_.[[".expr"]], envir = ._mirai_., enclos = NULL)))))
   sc <- sc[(length(sc) - 1L):(idx + 1L)]
   if (sc[[1L]][[1L]] == ".handleSimpleError")
     sc <- sc[-1L]
@@ -492,5 +528,6 @@ mk_mirai_error <- function(e, sc) {
   `class<-`(`attr<-`(msg, "stack.trace", sc), c("miraiError", "errorValue", "try-error"))
 }
 
+.miraiInterrupt <- `class<-`("", c("miraiInterrupt", "errorValue", "try-error"))
 .snapshot <- expression(mirai:::snapshot())
 .timedelay <- expression(nanonext::msleep(500L))
